//---------------------------------------------------------------------------
// Copyright (C) 2000 Dallas Semiconductor Corporation, All Rights Reserved.
//
// Permission is hereby granted, free of charge, to any person obtaining a
// copy of this software and associated documentation files (the "Software"),
// to deal in the Software without restriction, including without limitation
// the rights to use, copy, modify, merge, publish, distribute, sublicense,
// and/or sell copies of the Software, and to permit persons to whom the
// Software is furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included
// in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY,  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
// IN NO EVENT SHALL DALLAS SEMICONDUCTOR BE LIABLE FOR ANY CLAIM, DAMAGES
// OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
// ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
// OTHER DEALINGS IN THE SOFTWARE.
//
// Except as contained in this notice, the name of Dallas Semiconductor
// shall not be used except as stated in the Dallas Semiconductor
// Branding Policy.
//---------------------------------------------------------------------------
//
// owerr.c - Library functions for error handling with 1-Wire library
//
// Version: 1.00
//

#include <string.h>
#ifndef _WIN32_WCE
#include <stdio.h>
#endif
#include "ownet.h"

#ifndef SIZE_OWERROR_STACK
   #ifdef SMALL_MEMORY_TARGET
      //for small memory, only hole 1 error
      #define SIZE_OWERROR_STACK 1
   #else
      #define SIZE_OWERROR_STACK 10
   #endif
#endif

//---------------------------------------------------------------------------
// Variables
//---------------------------------------------------------------------------

// Error Struct for holding error information.
// In DEBUG, this will also hold the line number and filename.
typedef struct
{
   int owErrorNum;
#ifdef DEBUG
   int lineno;
   char *filename;
#endif
} owErrorStruct;

// Ring-buffer used for stack.
// In case of overflow, deepest error is over-written.
static owErrorStruct owErrorStack[SIZE_OWERROR_STACK];

// Stack pointer to top-most error.
static int owErrorPointer = 0;


//---------------------------------------------------------------------------
// Functions Definitions
//---------------------------------------------------------------------------
int owGetErrorNum(void);
int owHasErrors(void);
#ifdef DEBUG
   void owRaiseError(int,int,char*);
#else
   void owRaiseError(int);
#endif
#ifndef SMALL_MEMORY_TARGET
   void owPrintErrorMsg(FILE *);
   void owPrintErrorMsgStd();
   char *owGetErrorMsg(int);
#endif


//--------------------------------------------------------------------------
// The 'owGetErroNum' returns the error code of the top-most error on the
// error stack.  NOTE: This function has the side effect of popping the
// current error off the stack.  All successive calls to 'owGetErrorNum'
// will further clear the error stack.
//
// For list of error codes, see 'ownet.h'
//
// Returns:   int :  The error code of the top-most error on the stack
//
int owGetErrorNum(void)
{
   int i = owErrorStack[ owErrorPointer ].owErrorNum;
   owErrorStack[ owErrorPointer ].owErrorNum = 0;
   if(!owErrorPointer)
      owErrorPointer = SIZE_OWERROR_STACK - 1;
   else
      owErrorPointer = (owErrorPointer - 1);
   return i;
}

//--------------------------------------------------------------------------
// The 'owHasErrors' is a boolean test function which tests whether or not
// a valid error is waiting on the stack.
//
// Returns:   TRUE (1) : When there are errors on the stack.
//            FALSE (0): When stack's errors are set to 0, or NO_ERROR_SET.
//
int owHasErrors(void)
{
   if(owErrorStack[ owErrorPointer ].owErrorNum)
      return 1; //TRUE
   else
      return 0; //FALSE
}

#ifdef DEBUG
   //--------------------------------------------------------------------------
   // The 'owRaiseError' is the method for raising an error onto the error
   // stack.
   //
   // Arguments:  int err - the error code you wish to raise.
   //             int lineno - DEBUG only - the line number where it was raised
   //             char* filename - DEBUG only - the file name where it occured.
   //
   void owRaiseError(int err, int lineno, char* filename)
   {
      owErrorPointer = (owErrorPointer + 1) % SIZE_OWERROR_STACK;
      owErrorStack[ owErrorPointer ].owErrorNum = err;
      owErrorStack[ owErrorPointer ].lineno = lineno;
      owErrorStack[ owErrorPointer ].filename = filename;
   }
#else
   //--------------------------------------------------------------------------
   // The 'owRaiseError' is the method for raising an error onto the error
   // stack.
   //
   // Arguments:  int err - the error code you wish to raise.
   //
   void owRaiseError(int err)
   {
      owErrorPointer = (owErrorPointer + 1) % SIZE_OWERROR_STACK;
      owErrorStack[ owErrorPointer ].owErrorNum = err;
   }
#endif


// SMALL_MEMORY_TARGET - embedded microcontrollers, where these
// messaging functions might not make any sense.
#ifndef SMALL_MEMORY_TARGET
   //Array of meaningful error messages to associate with codes.
   //Not used on targets with low memory (i.e. PIC).
   static char *owErrorMsg[79] =
   {
      "No Error Was Set",
      "No Devices found on 1-Wire Network",
      "1-Wire Net Reset Failed",
      "Search ROM Error: Couldn't locate next device on 1-Wire",
      "Access Failed: Could not select device",
      "DS2480 Adapter Not Detected",
      "DS2480: Wrong Baud",
      "DS2480: Bad Response",
      "Open COM Failed",
      "Write COM Failed",
      "Read COM Failed",
      "Data Block Too Large",
      "Block Transfer failed",
      "Program Pulse Failed",
      "Program Byte Failed",
      "Write Byte Failed",
      "Read Byte Failed",
      "Write Verify Failed",
      "Read Verify Failed",
      "Write Scratchpad Failed",
      "Copy Scratchpad Failed",
      "Incorrect CRC Length",
      "CRC Failed",
      "Failed to acquire a necessary system resource",
      "Failed to initialize system resource",
      "Data too long to fit on specified device.",
      "Read exceeds memory bank end.",
      "Write exceeds memory bank end.",
      "Device select failed",
      "Read Scratch Pad verify failed.",
      "Copy scratchpad complete not found",
      "Erase scratchpad complete not found",
      "Address read back from scrachpad was incorrect",
      "Read page with extra-info not supported by this memory bank",
      "Read page packet with extra-info not supported by this memory bank",
      "Length of packet requested exceeds page size",
      "Invalid length in packet",
      "Program pulse required but not available",
      "Trying to access a read-only memory bank",
      "Current bank is not general purpose memory",
      "Read back from write compare is incorrect, page may be locked",
      "Invalid page number for this memory bank",
      "Read page with CRC not supported by this memory bank",
      "Read page with CRC and extra-info not supported by this memory bank",
      "Read back from write incorrect, could not lock page",
      "Read back from write incorrect, could not lock redirect byte",
      "The read of the status was not completed.",
      "Page redirection not supported by this memory bank",
      "Lock Page redirection not supported by this memory bank",
      "Read back byte on EPROM programming did not match.",
      "Can not write to a page that is locked.",
      "Can not lock a redirected page that has already been locked.",
      "Trying to redirect a locked redirected page.",
      "Trying to lock a page that is already locked.",
      "Trying to write to a memory bank that is write protected.",
      "Error due to not matching MAC.",
      "Memory Bank is write protected.",
      "Secret is write protected, can not Load First Secret.",
      "Error in Reading Scratchpad after Computing Next Secret.",
      "Load Error from Loading First Secret.",
      "Power delivery required but not available",
      "Not a valid file name.",
      "Unable to Create a Directory in this part.",
      "That file already exists.",
      "The directory is not empty.",
      "The wrong type of part for this operation.",
      "The max len for this file is too small.",
      "This is not a write once bank.",
      "The first directory page is not valid.",
      "The file can not be found.",
      "There is not enough space availabe.",
      "There is not a page to match that bit in the bitmap.",
      "There are no jobs for EPROM parts.",
      "Function not supported to modify attributes.",
      "Handle is not in use.",
      "Tring to read a write only file.",
      "There is no handle available for use.",
      "The directory provided is an invalid directory.",
      "Handle does not exist."
   };

   char *owGetErrorMsg(int err)
   {
      return owErrorMsg[err];
   }

   //--------------------------------------------------------------------------
   // The 'owPrintErrorMsg' is the method for printing an error from the stack.
   // The destination for the print is specified by the argument, fileno, which
   // can be stderr, stdout, or a log file.  In non-debug mode, the output is
   // of the form:
   // Error num: Error msg
   //
   // In debug-mode, the output is of the form:
   // Error num: filename line#: Error msg
   //
   // NOTE: This function has the side-effect of popping the error off the stack.
   //
   // Arguments:  FILE*: the destination for printing.
   //
   void owPrintErrorMsg(FILE *fileno)
   {
   #ifdef DEBUG
      int l = owErrorStack[ owErrorPointer ].lineno;
      char *f = owErrorStack[ owErrorPointer ].filename;
      int err = owGetErrorNum();
      fprintf(fileno,"Error %d: %s line %d: %s\r\n",err,f,l,owErrorMsg[err]);
   #else
      int err = owGetErrorNum();
      fprintf(fileno,"Error %d: %s\r\n",err,owErrorMsg[err]);
   #endif
   }

   // Same as above, except uses default printf output
   void owPrintErrorMsgStd()
   {
   #ifdef DEBUG
      int l = owErrorStack[ owErrorPointer ].lineno;
      char *f = owErrorStack[ owErrorPointer ].filename;
      int err = owGetErrorNum();
      printf("Error %d: %s line %d: %s\r\n",err,f,l,owErrorMsg[err]);
   #else
      int err = owGetErrorNum();
      printf("Error %d: %s\r\n",err,owErrorMsg[err]);
   #endif
   }
#endif

